"""What's the current version for each biological database?"""  # noqa:D400

from .sources import VersionFailure, clear_cache, get_rows, get_version, iter_versions, resolve
from .utils import VersionResult

__all__ = [
    "VersionFailure",
    "VersionResult",
    "clear_cache",
    "get_rows",
    "get_version",
    "iter_versions",
    "resolve",
]



"""A getter for the Molecular Oncology Almanac."""

from ..utils import Getter, VersionType, find, get_soup

__all__ = [
    "MOAlmanacGetter",
]


class MOAlmanacGetter(Getter):
    """A getter for MOAlmanac."""

    name = "Molecular Oncology Almanac"
    homepage_fmt = "https://github.com/vanallenlab/moalmanac-db/releases/tag/v.{version}"
    date_version_fmt = "%Y-%m-%d"
    version_type = VersionType.date

    def get(self) -> str:
        """Get the latest MOAlmanac version number."""
        soup = get_soup("https://moalmanac.org/")
        sub_footer = find(soup, "div", {"class": "text-right"})
        anchor = find(sub_footer, "a")
        version = anchor.text.strip("v").strip(".")
        return version


if __name__ == "__main__":
    MOAlmanacGetter.print()



"""A getter for BiGG."""

from datetime import datetime

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "BiGGGetter",
]

URL = "http://bigg.ucsd.edu/api/v2/database_version"


class BiGGGetter(Getter):
    """A getter for BiGG."""

    name = "BiGG"
    version_type = VersionType.semver

    def get(self):
        """Get the latest BiGG version number."""
        res = requests.get(URL, timeout=15).json()
        date = datetime.fromisoformat(res["last_updated"])
        return {
            "version": res["bigg_models_version"],
            "date": date,
        }


if __name__ == "__main__":
    BiGGGetter.print()



"""A getter for the Protein Ontology."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "PRGetter",
]

URL = "https://proconsortium.org/cgi-bin/sta_pro"


class PRGetter(Getter):
    """A getter for the Protein Ontology."""

    bioregistry_id = "pr"
    name = "Protein Ontology"
    version_type = VersionType.semver_minor
    date_fmt = "%m/%d/%Y"
    homepage_fmt = "https://proconsortium.org/download/release_{version}/"

    def get(self):
        """Get the latest Protein Ontology version number."""
        soup = get_soup(URL)
        rows = soup.find("table", **{"class": "nrm11"}).find_all("tr")
        row = list(rows)[2]
        version_cell, date_cell, *_ = list(row.find_all("td"))
        return {"version": version_cell.text.strip(), "date": date_cell.text.strip()}


if __name__ == "__main__":
    PRGetter.print()



"""A getter for ChemIDplus."""

import re

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "ChemIDplusGetter",
]


class ChemIDplusGetter(Getter):
    """A getter for ChemIDplus."""

    bioregistry_id = "chemidplus"
    name = "ChemIDplus"
    date_version_fmt = "%Y-%m-%d"
    homepage_fmt = "https://ftp.nlm.nih.gov/projects/chemidlease/chem.xml.{version}.zip"
    version_type = VersionType.date

    def get(self):
        """Get the latest ChemIDplus version number."""
        latest_url = "https://ftp.nlm.nih.gov/projects/chemidlease/CurrentChemID.xml"
        headers = {"Range": "bytes=0-300"}  # leave some slack to capture date
        r = requests.get(latest_url, headers=headers, timeout=15)
        if r.status_code == 206:
            result = re.search(r" date=\"([0-9]{4}-[0-9]{2}-[0-9]{2})\">", r.text)
            if result:
                return result.groups()[0]
        raise ValueError

    @staticmethod
    def homepage_version_transform(version: str) -> str:
        """Replace dots with dashes for ChemIDplus homepage format."""
        return version.replace("-", "")


if __name__ == "__main__":
    ChemIDplusGetter.print()



"""A getter for GuideToPharmacology."""

import re
from datetime import datetime
from typing import ClassVar

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "GuideToPharmacologyGetter",
]

URL = "https://www.guidetopharmacology.org/download.jsp"
RE = re.compile(r"^.*(\d{4}\.\d+).*(\d{2}\/\d{2}\/\d{2}).*$")


class GuideToPharmacologyGetter(Getter):
    """A getter for the IUPHAR Guide to Pharmacology."""

    name = "Guide to Pharmacology"
    homepage_fmt = "https://www.guidetopharmacology.org/DATA/public_iuphardb_v{version}.zip"
    date_fmt = "%Y-%m-%d"
    version_type = VersionType.year_minor
    collection: ClassVar[list[str]] = ["iuphar.family", "iuphar.ligand", "iuphar.receptor"]

    def get(self) -> dict[str, str]:
        """Get the latest Guide to Pharmacology version number."""
        soup = get_soup(URL)
        divs = list(soup.find_all("div", {"class": "contentboxfullhelp"}))
        # the type ignore is because mypy doesn't understand the attribute-based dispatch
        text = divs[4].div.ul.li.a.text  # type:ignore[attr-defined,union-attr]
        search = RE.search(text)
        if not search:
            raise ValueError(
                "Unable to extract version/date from Guide to Pharmacology Downloads page."
            )
        grps = search.groups()
        date = datetime.strftime(datetime.strptime(grps[1], "%d/%m/%y"), self.date_fmt)
        return {"version": grps[0], "date": date}


if __name__ == "__main__":
    GuideToPharmacologyGetter.print()



"""A getter for Cellosaurus."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "CellosaurusGetter",
]

URL = "https://ftp.expasy.org/databases/cellosaurus/cellosaurus.obo"


class CellosaurusGetter(Getter):
    """A getter for Cellosaurus."""

    bioregistry_id = "cellosaurus"
    name = "Cellosaurus"
    version_type = VersionType.sequential
    date_fmt = "%m:%d:%Y %H:%M"

    #   12:15:2022 12:00
    def get(self):
        """Get the latest Cellosaurus version number."""
        res = requests.get(URL, stream=True, timeout=15)
        data = {}
        for line in res.iter_lines(decode_unicode=True):
            line = line.strip().decode("utf8")
            if not line:
                break
            key, value = (part.strip() for part in line.split(":", 1))
            data[key] = value
        return {"version": data["data-version"], "date": data["date"]}


if __name__ == "__main__":
    CellosaurusGetter.print()



"""A getter for ITIS."""

import datetime

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "ITISGetter",
]

URL = "https://www.itis.gov/DisplayPresentDate"


class ITISGetter(Getter):
    """A getter for ITIS."""

    bioregistry_id = "itis"
    name = "ITIS"
    version_type = VersionType.date

    def get(self) -> datetime.datetime:
        """Get the latest ITIS version number."""
        res = requests.get(URL, timeout=3).text
        return datetime.datetime.strptime(res, "%d-%b-%Y")


if __name__ == "__main__":
    ITISGetter.print()



"""A getter for `SIGNOR <https://signor.uniroma2.it>`_."""

import datetime
from typing import ClassVar

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "SignorGetter",
]

URL = "https://signor.uniroma2.it/downloads.php"
TEXT = "Click here to download the latest stable release"


class SignorGetter(Getter):
    """A getter for SIGNOR."""

    name = "SIGNOR"
    version_type = VersionType.date
    date_fmt = "%B %Y"
    collection: ClassVar[list[str]] = ["signor", "signor.relation"]

    def get(self) -> datetime.datetime:
        """Get the latest SIGNOR version number."""
        soup = get_soup(URL)
        for p in soup.find_all("p"):
            if TEXT in p.text:
                _, _, after = p.text.partition(TEXT)
                after = after.strip().lstrip("(").rstrip(")")
                return datetime.datetime.strptime(after, "%B %Y")
        raise RuntimeError


if __name__ == "__main__":
    SignorGetter.print()



"""A getter for the NCI Thesaurus."""

import requests

from ..utils import Getter, VersionType

__all__ = [
    "NCItGetter",
]

URL = "https://evsexplore.semantics.cancer.gov/evsexplore/api/v1/metadata/terminologies"


class NCItGetter(Getter):
    """A getter for the NCI Thesaurus."""

    bioregistry_id = "ncit"
    name = "National Cancer Institute Thesaurus"
    date_fmt = "%B %d, %Y"
    version_type = VersionType.other

    def get(self) -> dict[str, str]:
        """Get the latest NCIt version number."""
        records = requests.get(URL, timeout=5).json()
        ncit_record = next(record for record in records if record["terminology"] == "ncit")
        return {
            "version": ncit_record["version"],
            "date": ncit_record["date"],
        }


if __name__ == "__main__":
    NCItGetter.print()



"""A getter for the Antibody Registry."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "AntibodyRegistryGetter",
]

URL = "https://www.antibodyregistry.org/api/datainfo"


class AntibodyRegistryGetter(Getter):
    """A getter for the Antibody Registry."""

    bioregistry_id = "antibodyregistry"
    name = "Antibody Registry"
    homepage_fmt = "https://antibodyregistry.org/"
    version_type = VersionType.date

    def get(self):
        """Get the latest Antibody Registry version number."""
        res = requests.get(URL, timeout=3)
        res_json = res.json()
        return res_json["lastupdate"]


if __name__ == "__main__":
    AntibodyRegistryGetter.print()



"""A getter for Complex Portal."""

from bioversions.utils import Getter, VersionType, _get_ftp_date_version

__all__ = [
    "ComplexPortalGetter",
]


class ComplexPortalGetter(Getter):
    """A getter for Complex Portal."""

    bioregistry_id = "complexportal"
    name = "Complex Portal"
    homepage_fmt = "https://ftp.ebi.ac.uk/pub/databases/intact/complex/{version}/"
    date_version_fmt = "%Y-%m-%d"
    version_type = VersionType.date

    def get(self):
        """Get the latest ComplexPortal version number."""
        return _get_ftp_date_version("ftp.ebi.ac.uk", "pub/databases/intact/complex/")


if __name__ == "__main__":
    ComplexPortalGetter.print()



"""A getter for GTDB."""

import requests

from bioversions.utils import Getter, VersionType

__all__ = [
    "GTDBGetter",
]

URL = "https://data.ace.uq.edu.au/public/gtdb/data/releases/latest/VERSION.txt"


class GTDBGetter(Getter):
    """A getter for the Genome Taxonomy Database (GTDB)."""

    bioregistry_id = "gtdb"
    name = "Genome Taxonomy Database"
    version_type = VersionType.sequential
    date_fmt = "%b %d, %Y"  # Format to match "Apr 24, 2024"
    homepage_fmt = "https://gtdb.ecogenomic.org/"

    def get(self):
        """Get the latest GTDB version number from VERSION.txt."""
        res = requests.get(URL, timeout=15)
        lines = res.text.strip().split("\n")

        # First line contains version like "v220"
        version_line = next(line for line in lines if line.startswith("v"))
        version = version_line.strip().lstrip("v")

        # Third line contains date like "Released Apr 24, 2024"
        date_line = next(line for line in lines if line.startswith("Released "))
        date = date_line.strip().removeprefix("Released ")

        return {"version": version, "date": date}


if __name__ == "__main__":
    GTDBGetter.print()



"""A getter for ZFIN."""

from datetime import datetime

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "ZfinGetter",
]

URL = "https://zfin.org/downloads"


class ZfinGetter(Getter):
    """A getter for ZFIN."""

    bioregistry_id = "zfin"
    name = "Zebrafish Information Network"
    version_type = VersionType.date

    def get(self):
        """Get the latest ZFIN version number."""
        soup = get_soup(URL)
        header = soup.find("h2")
        version = header.text[len("ZFIN Data Reports from: ") :].strip()
        return datetime.strptime(version, "%d %b %Y").strftime("%Y-%m-%d")


if __name__ == "__main__":
    ZfinGetter.print()



"""A getter for NPASS."""

from bioversions.utils import Getter, VersionType, find, get_soup

__all__ = [
    "NPASSGetter",
]

URL = "http://bidd.group/NPASS/"


class NPASSGetter(Getter):
    """A getter for NPASS."""

    bioregistry_id = "npass"
    name = "NPASS"
    version_type = VersionType.semver_minor

    def get(self) -> str:
        """Get the latest NPASS version number."""
        return "2.0"


def _dynamic_get() -> str:
    # this has been retired since the website is so slow and this
    # resource is effectively static
    soup = get_soup(URL)
    footer = find(soup, name="footer")
    ul = find(footer, name="ul")
    for li in ul.find_all(name="li"):
        if li.text.startswith("Version:"):
            return li.text[len("Version: ") :]
    raise ValueError(f"could not parse NPASS version from {URL}")


if __name__ == "__main__":
    NPASSGetter.print()



"""A getter for the Mouse Genome Database."""

from bioversions.utils import Getter, VersionType, get_soup

__all__ = [
    "MGIGetter",
]


HOMEPAGE = "https://www.informatics.jax.org/"


class MGIGetter(Getter):
    """A getter for MGI."""

    bioregistry_id = "mgi"
    name = "Mouse Genome Database"
    version_type = VersionType.semver_minor

    def get(self) -> str:
        """Get the latest MGI version number."""
        soup = get_soup(HOMEPAGE)
        cells = soup.find_all("td")
        search_string = "MGI"
        for cell in cells:
            text = cell.text
            if text and text.strip().startswith("last database update"):
                idx = text.find(search_string)
                return text[idx + len(search_string) :].strip()
        raise ValueError


if __name__ == "__main__":
    MGIGetter.print()
